/* This example code is placed in the Public Domain. */

/* Parse a drawing written in s-expression into a compacted dom and
   "draw" it. Cache the floating point value of each node (meaningful for
   coordinates) to demonstrate how to extend the node structure by wrapping it.
*/

#include <stdlib.h>
#include <stdio.h>
#include "string.h"
#include <gensexpr/gsxl.h>

/* Wrapper structure for a node; this allow us storing extra data for
   each node, e.g. the floating point value of the node */
typedef struct {
	gsxl_node_t hdr;
	double val;
} my_node_t;

/* Return the idxth item on the list as a string (or NULL) */
static const char *arg_str(my_node_t *node, int idx)
{
	node = (my_node_t *)gsxl_nth(&node->hdr, idx);

	if (node == NULL)
		return NULL;
	return node->hdr.str;
}

/* Return the idxth item on the list as a double (or 0.0) */
static double arg_double(my_node_t *node, int idx)
{
	node = (my_node_t *)gsxl_nth(&node->hdr, idx);
	if (node == NULL)
		return 0.0;
	return node->val;
}

/* Expect pt children, print coords */
static void draw_poly_outline(const char *type, my_node_t *node)
{
	my_node_t *n;
	printf("draw polygon %s:", type);
	for(n = node; n != NULL; n = gsxl_next(n)) { /* points is a list */
		if ((n->hdr.str == NULL) || (strcmp(n->hdr.str, "pt") != 0)) /* deal with pt commands only */
			continue;
		printf(" %.2f;%.2f", arg_double(n, 1), arg_double(n, 2));
	}
	printf("\n");
}

/* Expect color, contour and hole children, print "drawing commands" */
static void draw_poly(my_node_t *node)
{
	my_node_t *n;
	for(n = node; n != NULL; n = gsxl_next(n)) {
		if (n->hdr.str == NULL) /* ignore empty command */
			continue;
		if (strcmp(n->hdr.str, "color") == 0)
			printf("Set color to %s\n", arg_str(n, 1));
		else if (strcmp(n->hdr.str, "contour") == 0)
			draw_poly_outline("contour", gsxl_children(n));
		else if (strcmp(n->hdr.str, "hole") == 0)
			draw_poly_outline("hole", gsxl_children(n));
	}
}

/* Draw the file recursively */
static void draw(my_node_t *node)
{
	if (node->hdr.str == NULL)
		return;
	if (strcmp(node->hdr.str, "poly") == 0)
		draw_poly(gsxl_children(node));
}

/* Calculate and cache double values in each node of the tree */
static void parse_vals(my_node_t *node)
{
	node->val = strtod(node->hdr.str, NULL);
	for(node = gsxl_children(node); node != NULL; node = gsxl_next(node))
		parse_vals(node);
}


int main(int argc, char *argv[])
{
	int c;
	gsx_parse_res_t res;
	gsxl_dom_t dom;

	/* Parse input */
	gsxl_init(&dom, my_node_t);
	do {
		c = getchar();
	} while((res = gsxl_parse_char(&dom, c)) == GSX_RES_NEXT);

	if (res == GSX_RES_EOE) {
		/* compact and simplify the tree */
		gsxl_compact_tree(&dom);

		/* recursively calculate the double value of each node */
		parse_vals((my_node_t *)dom.root);

		/* recursively draw the file */
		draw((my_node_t *)dom.root);
	}

	/* clean up */
	gsxl_uninit(&dom);
	return 0;
}
